@@ -67,7 +67,7 @@ typedef struct GlobalOptionsTag
     /** Size of 'corner' added to note boxes. */
     unsigned int noteCorner;
 
-    /** Anguluar box slope in pixels. */
+    /** Angular box slope in pixels. */
     unsigned int aboxSlope;
 
     /** If TRUE, wrap arc text as well as box contents. */
@@ -300,7 +300,7 @@ static char *splitStringToWidth(Context *ctx, char *l, unsigned int width)
 
         /* Copy the remaining line to the start of the string */
         m = 0;
-        n = (p - l);
+        n = (int)(p - l);
 
         while (isspace(orig[n]) && orig[n] != '\0')
         {
@@ -390,7 +390,7 @@ static unsigned int computeLabelLines(Context          *ctx,
         char *nextLine = strstr(label, "\\n");
         if (nextLine)
         {
-            const int lineLen = nextLine - label;
+            const int lineLen = (int)(nextLine - label);
 
             /* Allocate storage and duplicate the line */
             retLines[c] = malloc_s(lineLen + 1);
@@ -633,11 +633,11 @@ static char *getLine(const char        *string,
     /* Determine the length of the line */
     if(lineEnd != NULL)
     {
-        lineLen = lineEnd - lineStart;
+        lineLen = (unsigned int)(lineEnd - lineStart);
     }
     else
     {
-        lineLen = strlen(string) - (lineStart - string);
+        lineLen = (unsigned int)(strlen(string) - (lineStart - string));
     }
 
     /* Clamp the length to the buffer */
@@ -662,7 +662,7 @@ static char *getLine(const char        *string,
  * \param  x           The x position at which the entity text should be centered.
  * \param  y           The y position where the text should be placed.
  * \param  entLabel    The label to render, which maybe \a NULL in which case
- *                       no ouput is produced.
+ *                       no output is produced.
  * \param  entUrl      The URL for rendering the label as a hyperlink.  This
  *                       maybe \a NULL if not required.
  * \param  entId       The text identifier for the arc.
@@ -704,7 +704,7 @@ static void entityText(Context          *ctx,
             /* Check if a URL is associated */
             if(entUrl)
             {
-                /* If no explict colour has been set, make URLS blue */
+                /* If no explicit colour has been set, make URLS blue */
                 ctx->drw.setPen(&ctx->drw, ADRAW_COL_BLUE);
 
                 /* Image map output */
@@ -1062,7 +1062,7 @@ static void arcBox(Context           *ctx,
  * \param arcIdUrl       The URL for rendering the test identifier as a hyperlink.
  *                        This maybe \a NULL if not required.
  * \param arcTextColour  Colour for the arc text, or NULL to use default.
- * \param arcTextColour  Colour for the arc text backgroun, or NULL to use default.
+ * \param arcTextColour  Colour for the arc text background, or NULL to use default.
  * \param arcType        The type of arc, used to control output semantics.
  */
 static void arcText(Context           *ctx,
@@ -1328,8 +1328,8 @@ static void arcLine(Context          *ctx,
             hasArrows = FALSE;
 
             /* Get co-ordinates of the arc end-point */
-            ADrawComputeArcPoint(sx, y - 1, ctx->opts.entitySpacing - 8,
-                                 ctx->opts.loopArcHeight, 180 - 45,
+            ADrawComputeArcPoint((float)sx, (float)(y - 1), (float)(ctx->opts.entitySpacing - 8),
+                                 (float)ctx->opts.loopArcHeight, 180.0f - 45.0f,
                                  &px, &py);
 
             /* Draw a cross */
@@ -1392,8 +1392,9 @@ static void arcLine(Context          *ctx,
             hasArrows = FALSE;
 
             /* Get co-ordinates of the arc end-point */
-            ADrawComputeArcPoint(sx, y - 1, ctx->opts.entitySpacing - 8,
-                                 ctx->opts.loopArcHeight, 45,
+            ADrawComputeArcPoint((float)sx, (float)(y - 1),
+                                 (float)(ctx->opts.entitySpacing - 8),
+                                 (float)ctx->opts.loopArcHeight, 45.0f,
                                  &px, &py);
 
             /* Draw a cross */
@@ -1431,7 +1432,7 @@ static void arcLine(Context          *ctx,
  */
 static Boolean checkMsc(Msc m)
 {
-    /* Check all arc entites are known */
+    /* Check all arc entities are known */
     MscResetArcIterator(m);
     do
     {
@@ -1710,7 +1711,7 @@ int mscgen_generate(const char *inputFile,
         assert(startCol != -1);
         assert(endCol != -1 || isBroadcastArc(MscGetCurrentArcDest(m)));
 
-        /* Check for entity colouring if not set explicity on the arc */
+        /* Check for entity colouring if not set explicitly on the arc */
         if (arcTextColour == NULL)
         {
           arcTextColour = MscGetEntAttrib(m, startCol, MSC_ATTR_ARC_TEXT_COLOUR);
